/*
 * YAG -- Yet Another Grapher
 *	$Revision: 0.977 $
 * A Jiffy Script, Inc. production
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "lib/copyright.h"
#include "lib/jiffy_err.h"

#ifndef YAG_VERSION
#define YAG_VERSION "0.99"
#endif

#define XMAX 1.0
#define XMIN 0.0
#define YMAX 1.0
#define YMIN 0.0
#define PGHEIGHT 11.0
#define PGWIDTH 8.5

#ifndef TRUE
#define TRUE 1
#endif
#define FALSE 0
#define NOT_SET	-1

void setup(char *, char *, char *, char *, float, float, float, float, int, int, 
	   int, int, int, FILE *, char *, int, int, char, int, char, float, float, float);
float fit(int, int, float *, float, float);
int check_flag(char *);
void usage(char *);
void make_graph(FILE *, FILE *, float, float, int, char *, int, int, char *, 
		int, int, char, char, int, int);
char **get_my_argv(char **, FILE *, int *);
char *get_first_line(FILE *);
int get_form(char *, int *, char *, int *, char *);
extern int jgetopt(int, char **, char *);
extern char *jstrdup(char *);

void
main(int argc, char **argv)
{
  int i;
  float j;
  int ac;

  char *xlabel;
  int xlabel_count = FALSE;
  char *ylabel;
  int ylabel_count = FALSE;
  char *title;
  int title_count = FALSE;
  char *subtitle;
  int subtitle_count = FALSE;
  int input_count = FALSE;
  FILE *input_fp = stdin;
  int output_count = FALSE;
  FILE *output_fp = stdout;
  char *fontname;
  int font = NOT_SET;
  char *graph_label;
  int graph_count = FALSE;
  int xtick = 0;
  int ytick = 0;
  float xmin;
  float xmax;
  float ymin;
  float ymax;
  float page_height;
  float page_width;
  int ph_count = FALSE;
  int pw_count = FALSE;
  int xmin_count = FALSE;
  int xmax_count = FALSE;
  int ymax_count = FALSE;
  int ymin_count = FALSE;
  int epsf = NOT_SET;		/* do they want epsf output? */
  int grid = NOT_SET;
  int equal = NOT_SET;
  float xdif;
  float ydif;
  int LANDSCAPE = NOT_SET;
  int color = NOT_SET;
  int shade_graph = NOT_SET;
  int connect_lines = NOT_SET;
  int make_dots = NOT_SET;
  float thickness = 0;
  char *progname = argv[0];
  char *arglist1 = "input: output help xlabel ylabel xmin "
                   "xmax ymin ymax xtick ytick title subtitle landscape "
                   "nolandscape epsf grid nogrid equal noequal version font glabel "
	           "shade noshade color nocolor pageheight pagewidth lines no lines dots nodots thicker "
                   "thinner";
                                /* this is for parsing command-line args. */
  char *arglist2 = "input: output: help xlabel: ylabel: xmin: xmax: ymin: ymax: "
                   "xtick: ytick: title: subtitle: landscape nolandscape "
		   "epsf grid nogrid equal noequal font: glabel: shade noshade "
		   "color nocolor pageheight: pagewidth: lines nolines dots nodots thicker thinner";
                                /* this is for parsing command-line and input-file args. */

  char **my_argv;	/* first we do this with arrays, then do again with pointers */
  char **ta_argv;
  char *first_data_point;
  char *tmp_f_data;
  char form_string[128];
  char x_form;
  char y_form;
  int x_precis;
  int y_precis;

  extern int joptind;
  extern char *joptarg;

  int ta_size = 256;

  ta_argv = (char **) malloc(ta_size * (sizeof(int)));
  
  my_argv = ta_argv;

  ac = 0;



  if (argc > 1)
    {
      int cf;

      while ((cf = jgetopt(argc, argv, arglist1)) != EOF)
	{
	  switch (cf) 
	    {
	    case 2:
	      usage(progname);
	      exit(0);
	      break;
	    case 0:
	      if (input_count)
		{
		  fprintf(stderr, "An input file is already open.\n");     /* ITS4: ignore */
		}
	      else
		{
		  input_count = TRUE;
		  input_fp = fopen(joptarg, "r");                          /* ITS4: ignore */
		  if (!input_fp)
		    {
		      fprintf(stderr, "Could not open input file--defaulting to stdin\n");  /* ITS4: ignore */
		      input_fp = stdin;
		      input_count--;
		    }
		}
	      break;
	    case 20:
	      fprintf(stderr, "Version %s\n", YAG_VERSION);               /* ITS4: ignore */
	      exit(0);
	      break;
	    default:
	      *ta_argv++ = jstrdup(argv[joptind - 1]);
	      ac++;
	      break;
	    }
	}
    }


  my_argv = get_my_argv(my_argv, input_fp, &ac);

  /* parse the arguments */
  
  if (ac >= 1)
    {
      int cf;
      
      joptind = 0;
      while ((cf = jgetopt(ac, my_argv, arglist2)) != EOF) {
	switch (cf) {
	case 3:					/* -xlabel */
	  if (!xlabel_count) 
	    {
	      xlabel = (char *) malloc(strlen(joptarg));
	      strcpy(xlabel, joptarg);                                    /* ITS4: ignore */
	      xlabel_count = TRUE;
	    } 
	  break;
	case 4:					/* -ylabel */
	  if (!ylabel_count)
	    {
	      ylabel = (char *) malloc(strlen(joptarg));
	      strcpy(ylabel, joptarg);                                    /* ITS4: ignore */
	      ylabel_count = TRUE;
	    } 
	  break;
	case 11:				/* -title */
	  if (!title_count)
	    {
	      title = (char *) malloc(strlen(joptarg));
	      strcpy(title, joptarg);                                    /* ITS4: ignore */
	      title_count = TRUE;
	    }		  
	  break;
	case 12:				/* -subtitle */
	  if (!subtitle_count)
	    {
	      subtitle = (char *) malloc(strlen(joptarg));
	      strcpy(subtitle, joptarg);                                 /* ITS4: ignore */
	      subtitle_count = TRUE;
	    }		  
	  break;
	case 13:				/* -landscape */
	  if (LANDSCAPE == NOT_SET)
	    {
	      LANDSCAPE = TRUE;
	    }
	  break;
	case 14:				/* -nolandscape */
	  if (LANDSCAPE == NOT_SET)
	    {
	      LANDSCAPE = FALSE;
	    }
	  break;
	case 15:				/* -epsf */
	  if (epsf == NOT_SET)
	    {
	      epsf = TRUE;
	    }
	  break;
	case 6:					/* -xmax */
	  if (!xmax_count)
	    {
	      if (*joptarg == '-')
		{
		  *++joptarg;
		  sscanf(joptarg, "%f", &xmax);
		  xmax = xmax * (-1.0);
		} /* the number is neg */
	      else
		{
		  sscanf(joptarg, "%f", &xmax);
		}
	      xmax_count = TRUE;
	    }		 
	  break;
	case 5:					/* -xmin */
	  if (!xmin_count)
	    {
	      if (*joptarg == '-')
		{
		  *++joptarg;
		  sscanf(joptarg, "%f", &xmin);
		  xmin = xmin * (-1.0);
		} /* the number is neg */
	      else
		{
		  sscanf(joptarg, "%f", &xmin);
		}
	      xmin_count = TRUE;
	    }		  
	  break;
	case 8:					/* -ymax */
	  if (!ymax_count)
	    {
	      if (*joptarg == '-')
		{
		  *++joptarg;
		  sscanf(joptarg, "%f", &ymax);
		  ymax = ymax * (-1.0);
		} /* the number is neg */
	      else
		{
		  sscanf(joptarg, "%f", &ymax);
		}
	      ymax_count = TRUE;
	    }		 
	  break;
	case 7:					/* -ymin */
	  if (!ymin_count)
	    {
	      if (*joptarg == '-')
		{
		  *++joptarg;
		  sscanf(joptarg, "%f", &ymin);
		  ymin = ymin * (-1.0);
		} /* the number is neg */
	      else
		{
		  sscanf(joptarg, "%f", &ymin);
		}
	      ymin_count = TRUE;
	    }		  
	  break;
	case 9:					/* -xtick */
	  if (!xtick && isdigit(*joptarg))
	    {
	      xtick = atoi(joptarg);
	    }
	  break;
	case 10:				/* -ytick */
	  if (!ytick && isdigit(*joptarg))
	    {
	      ytick = atoi(joptarg);
	    }
	  break;
	case 16:				/* -grid */
	  if (grid == NOT_SET)
	    {
	      grid = TRUE;
	    }
	  break;
	case 17:				/* -nogrid */
	  if (grid == NOT_SET)
	    {
	      grid = FALSE;
	    }
	  break;
	case 18:				/* -equal */
	  if (equal == NOT_SET)
	    {
	      equal = TRUE;	
	    }
	  break;
	case 19:				/* -noequal */
	  if (equal == NOT_SET)
	    {
	      equal = FALSE;
	    }
	  break;
	case 20:				/* -font */
	  if (font == NOT_SET)
	    {
	      fontname = (char *) malloc(strlen(joptarg));
	      strcpy(fontname, joptarg);                                /* ITS4: ignore */
	      font = TRUE;
	    }
	  break;
	case 21:				/* -glabel */
	  if (!graph_count)
	    {
	      graph_label = (char *) malloc(strlen(joptarg));
	      strcpy(graph_label, joptarg);                             /* ITS4: ignore */
	      graph_count = TRUE;
	    }
	  break;
	case 22:				/* -shade */
	  if (shade_graph == NOT_SET)
	    {
	      shade_graph = TRUE;
	    }
	  break;
	case 23:				/* -noshade */
	  if (shade_graph == NOT_SET)
	    {
	      shade_graph = FALSE;
	    }
	  break;
	case 24:				/* -color */
	  if (color == NOT_SET)
	    {
	      color = TRUE;
	    }
	  break;
	case 25:				/* -nocolor */
	  if (color == NOT_SET)
	    {
	      color = FALSE;
	    }
	  break;
	case 26:				/* -pageheight */
	  if (!ph_count)
	    {
	      if (*joptarg == '-')
		{
		  *++joptarg;
		} /* the number is neg */
	      sscanf(joptarg, "%f", &page_height);
	      ph_count = TRUE;
	    }		  
	  break;
	case 27:				/* -pagewidth */
	  if (!pw_count)
	    {
	      if (*joptarg == '-')
		{
		  *++joptarg;
		} /* the number is neg */
	      sscanf(joptarg, "%f", &page_width);
	      pw_count = TRUE;
	    }		  
	  break;
	case 28:				/* -lines */
	  if (connect_lines == NOT_SET)
	    {
	      connect_lines = TRUE;
	    }
	  break;
	case 29:				/* -nolines */
	  if (connect_lines == NOT_SET)
	    {
	      connect_lines = FALSE;
	    }
	  break;
	case 30:				/* -dots */
	  if (make_dots == NOT_SET)
	    {
	      make_dots = TRUE;
	    }
	  break;
	case 31:				/* -nodots */
	  if (make_dots == NOT_SET)
	    {
	      make_dots = FALSE;
	    }
	  break;
	case 32:				/* -thicker */
	  if (!thickness)
	    {
	      thickness = .001;
	    }
	  break;
	case 33:				/* -thinner */
	  if (thickness)
	    {
	      thickness = 0;
	    }
	  break;
	case 1:					/* -output */
	  if (!output_count)
	    {
	      output_count = TRUE;
	      
	      output_fp = fopen(joptarg, "w");
	      if (!output_fp)
		{
		  fprintf(stderr, "could not open file %s\n", joptarg);
		}
	    }		  
	  else
	    {
	      fprintf(stderr, "You already have named an output file.\n");   /* ITS4: ignore */
	    }
	  break;
	case JNOT_IN_LIST:
	  fprintf(stderr, "The flag (%s) is not available\n", my_argv[joptind - 1]);
	  usage(progname);
	  exit(cf);
	  break;
	default:
	  fprintf(stderr, "Over-ridden flag (%s)\n", my_argv[joptind - 1]);
	  break;
	}
      }
    }


  /* set the defaults if they have not been set by the user */

  if (!xlabel_count)
    {
      xlabel = "X-axis";
    }
  if (!ylabel_count)
    {
      ylabel = "Y-axis";
    }
  if (!title_count)
    {
      title = "YAG graph";
    }
  if (!subtitle_count)
    {
      subtitle = " ";
    }
  if (font == NOT_SET)
    {
      fontname = "anyfont";
    }
  if (!graph_count)
    {
      graph_label = "\0";
    }
  if (!xmax_count)
    {
      xmax = XMAX;
    }
  if (!xmin_count)
    {
      xmin = XMIN;
    }
  if (!ymax_count)
    {
      ymax = YMAX;
    }
  if (!ymin_count)
    {
      ymin = YMIN;
    }
  if (!ph_count)
    {
      page_height = PGHEIGHT;
    }
  if (!pw_count)
    {
      page_width = PGWIDTH;
    }
  if (!xtick)
    {
      xtick = 25;
    }
  if (!ytick)
    {
      ytick = 25;
    }
  if (epsf == NOT_SET)
    {
      epsf = FALSE;
    }
  if (grid == NOT_SET)
    {
      grid = FALSE;
    }
  if (equal == NOT_SET)
    {
      equal = FALSE;
    }
  if (LANDSCAPE == NOT_SET)
    {
      LANDSCAPE = TRUE;
    }
  if (shade_graph == NOT_SET)
    {
      shade_graph = TRUE;
    }
  if (color == NOT_SET)
    {
      color = FALSE;
    }
  if (make_dots == NOT_SET)
    {
      make_dots = FALSE;
    }
  if (connect_lines == NOT_SET)
    {
      connect_lines = TRUE;
    }

  xdif = xmax - xmin;
  ydif = ymax - ymin;


/*
 * Read the first line of the input, determine the precision and
 *	the type (f, e, E) of the input.
 */

  first_data_point = get_first_line(input_fp);

  tmp_f_data = jstrdup(first_data_point);		/* because get_form() is destructive */

  get_form(tmp_f_data, &y_precis, &y_form, &x_precis, &x_form);

/* set up the POSTSCRIPT stuff */
  setup(xlabel, ylabel, title, subtitle, xmax, xmin, ymax, ymin, xtick, 
	ytick, LANDSCAPE, epsf, equal, output_fp, fontname, color, x_precis,
	x_form, y_precis, y_form, page_height, page_width, thickness);

/* make the axes */

  fprintf(output_fp, "/%s findfont 7 XScale div scalefont setfont\n", fontname);

/* make the form_string for printing the Y-axis tick labels */
  sprintf(form_string, "YTLO neg %%d %%d div .002 sub m (%%.*%c) s\n", y_form);

  for (i = ytick; i >= 0 ; i--)
    {
      if (grid)
	{
	  fputs(".5 sg\n", output_fp);
	  fprintf(output_fp, "0 %d %d div m 1 %d %d div l st\n", i, ytick, i, ytick);
	  fputs("0 sg\n", output_fp);
	}
      else
	{
	  if (i == ytick)
	    {
	      fputs("0 sg\n", output_fp);
	    }
	  fprintf(output_fp, ".003 neg %d %d div m .003 %d %d div l st\n", i, ytick, i,
		  ytick);
	}
      fprintf(output_fp, form_string, i, ytick, y_precis,
	      ymin + (ydif * ((float) i / (float) ytick)));
    }

/* make the form_string for printing the X-axis tick labels */
  sprintf(form_string, "%%d %%d (%%.*%c) lx\n", x_form);

  for (i = xtick; i >= 0; i--)
    {
      if (grid)
	{
	  fputs(".5 sg\n", output_fp);
	  fprintf(output_fp, "%d %d div 0 m %d %d div 1 l st\n", i, xtick,
		  i, xtick);
	  fputs("0 sg\n", output_fp);
	}
      else
	{
	  if (i == xtick)
	    {
	      fputs("0 sg\n", output_fp);
	    }
	  fprintf(output_fp, "%d %d div .004 neg m %d %d div .004 l st\n", i, xtick,
		  i, xtick);
	}

      j = xmin + (xdif * ((float) i / (float) xtick));

      fprintf(output_fp, form_string, i, xtick, x_precis, j);
    }

  if (!grid)
    fputs("0 0 m 0 1 l st 0 0 m 1 0 l st\n", output_fp);

  fputs("gsave 0 0 m 0 1 l 1 1 l 1 0 l closepath clip newpath\n", output_fp);

  make_graph(input_fp, output_fp, xmax, xmin, shade_graph, graph_label, 0, 0, first_data_point,
	     x_precis, y_precis, x_form, y_form, connect_lines, make_dots);

  fputs("grestore\n", output_fp);

  if (!epsf)
    fputs("showpage\n", output_fp);

  fclose(input_fp);
  exit (0);
}


void
setup(char *xlabel, char *ylabel, char *title, char *subtitle, 
      float xmax, float xmin, float ymax, float ymin, int xtick, int ytick,
      int landscape, int epsf, int equal, FILE *output_fp, char *fontname,
      int color, int x_precis, char x_form, int y_precis, char y_form, 
      float page_height, float page_width, float thickness)
{
  float xdim;
  float ydim;
  char tlo_string[128];

  if (!epsf)
    {
      fputs("%!PS\n", output_fp);
    }
  else
    {
      fputs("%!PS-Adobe-3.0 EPSF-3.0\n", output_fp);	/* in accordance with the standard */
      if (landscape)
	{
/*	  fputs("%%BoundingBox: 8 18 725 545\n", output_fp);*/
	  fprintf(output_fp, "%%%%BoundingBox: 8 18 %f %f\n", 
	          (page_height - 1.0) * 72,
		  (page_width - 1.0) * 72 );
	  fputs("%%Creator: Jiffy Script Inc.' YAG\n", output_fp);
	  fprintf(output_fp, "%%Title: %s\n", title);
	}
      else
	{
/*	  fprintf(output_fp, "%%%%BoundingBox: 22 8 547 684\n", */
	  fprintf(output_fp, "%%%%BoundingBox: 22 8 %f %f\n", 
		  (page_width - 1.0) * 72,
	          (page_height - 1.0) * 72);
	  fputs("%%Creator: Jiffy Script Inc.'s YAG\n", output_fp);
	  fprintf(output_fp, "%%Title: %s\n", title);
	}
    }

  if (landscape)
    {
      xdim = (page_height - 2.0);
      if (!equal)
	{
	  ydim = (page_width - 2.0);
	}
      else
	{
	  ydim = fit(xtick, ytick, &xdim, xdim, (page_width - 2.0));
	}
    }
  else 
    {
      ydim = (page_height - 2.5);
      if (!equal)
	{
	  xdim = (page_width - 2.0);
	}
      else
	{
	  xdim = fit(ytick, xtick, &ydim, ydim, (page_width - 2.0));
	}
    }

  fprintf(output_fp, "/YScale %f 72 mul def\n", ydim);
  fprintf(output_fp, "/XScale %f 72 mul def\n", xdim);
  fprintf(output_fp, "/GlO 5 YScale div def\n");

  fprintf(output_fp, "/%s findfont 7 XScale div scalefont setfont\n", fontname);

  sprintf(tlo_string, "/XTLO {(%%.*%c) stringwidth pop 3 YScale div add} def\n", x_form);
  fprintf(output_fp, tlo_string, x_precis, (xmax > xmin) ? xmax : xmin);
  fprintf(output_fp, "/XLO XTLO %f %d mul YScale div add def\n", ydim, (ydim > xdim) ? 3 : 6);
  fprintf(output_fp, "/XAO XTLO YScale mul 28 sub def \n");              /* ITS4: ignore */
  fputs("XAO 0 lt {/XAO 28 def} if\n", output_fp);

  sprintf(tlo_string, "/YTLO {(%%.*%c) stringwidth pop 3 XScale div add} def\n", y_form);
  fprintf(output_fp, tlo_string, y_precis, (ymax > ymin) ? ymax : ymin);
  fprintf(output_fp, "/YLO YTLO %f %d mul 9 sub XScale div add def\n", xdim,  (xdim > ydim) ? 3 : 6);
  fprintf(output_fp, "/YAO YTLO XScale mul 28 sub def \n");              /* ITS4: ignore */
  fputs("YAO 0 lt {/YAO 28 def} if\n", output_fp);

/* make a function to convert reasonable fractions to our new scale */

  fprintf(output_fp, "/xmin %.2f def /xmax %.2f def\n", xmin, xmax);
  fputs("/deltax xmax xmin sub def\n", output_fp);
  fputs("/convx {xmin sub deltax div} def\n", output_fp);

/* 
 * PS stuff to only do color if the printing device is capable of doing so.
 * It is probably faulty, as the Red Book says not to use deviceinfo if one 
 * wants portablity.
 */

  if (color == TRUE)
    {
/*      fprintf(output_fp, "deviceinfo /Colors known\n" */                      /* ITS4: ignore */
/*	      "{ deviceinfo /Colors get 1 ne\n"
	      "{/sg {1 1 sethsbcolor} bind def}\n"
	      "{/sg {setgray} bind def} ifelse}\n"
	      "{/sg {setgray} bind def} ifelse\n");
*/
	fprintf(output_fp, "/sg {1 1 sethsbcolor} bind def\n");
    }
  else
    {
      fprintf(output_fp, "/sg {setgray} bind def\n");                      /* ITS4: ignore */
    }

/*  fputs("/sg {setgray} bind def\n", output_fp);*/
  fputs("/lx {/value exch def /tick exch def /loc exch def \n", output_fp);
  fputs("0 sg gsave 90 rotate XTLO neg loc tick div .005 add neg m \n", output_fp);
  fputs("value show grestore} bind def\n", output_fp);

  fprintf(output_fp, "/ymin %.2f def /ymax %.2f def\n", ymin, ymax);
  fputs("/deltay ymax ymin sub def\n", output_fp);
  fputs("/convy {ymin sub deltay div} def\n", output_fp);

  fputs("/c {gsave currentpoint .002 0 360 arc stroke grestore} bind def\n", output_fp);
  fputs("/l {lineto} bind def\n", output_fp);
  fputs("/s {show} bind def\n", output_fp);
  fputs("/st {stroke} bind def\n", output_fp);
  fputs("/m {moveto} bind def\n", output_fp);

  if (epsf || !landscape)
    {
      fputs("72 YAO add 72 XAO add translate\n", output_fp);
      fputs("/YScale YScale XAO sub def\n", output_fp);
      fputs("/XScale XScale YAO sub def\n", output_fp);
      fputs("/TAO {YAO sub} def\n", output_fp);
    }
  else
    {
      fputs("7.5 72 mul XAO sub 72 YAO add translate\n", output_fp);
      fputs("90 rotate\n", output_fp);
      fputs("/YScale YScale YAO sub def\n", output_fp);
      fputs("/XScale XScale XAO sub def\n", output_fp);
      fputs("/TAO {XAO sub} def\n", output_fp);
    }
  
  fprintf(output_fp, "XScale YScale scale\n");                              /* ITS4: ignore */

  fprintf(output_fp, "%f setlinewidth\n", thickness);

  fprintf(output_fp, "/%s findfont 24.5 XScale div scalefont setfont\n", fontname);
  
  fprintf(output_fp, "gsave 90 rotate 187 YScale div YLO m (%s) s grestore\n", ylabel);

  fprintf(output_fp, "194 XScale div XLO neg m (%s) s\n", xlabel);
/*  fprintf(output_fp, "194 XScale div 45 YScale div neg m (%s) s\n", xlabel);*/
/*  fprintf(output_fp, "129 XScale div %f TAO YScale div m (%s) s\n", ((ydim + 0.5) * 72.0), title); */
  fprintf(output_fp, "%f 2 div 72 mul XScale div (%s) stringwidth pop 2 div sub %f TAO YScale div m (%s) s\n", 
	  xdim, title, ((ydim + 0.5) * 72.0), title);
  fprintf(output_fp, "/%s findfont 20 XScale div scalefont setfont\n", fontname);
/*  fprintf(output_fp, "149 XScale div %f TAO YScale div m (%s) s\n", ((ydim + 0.25) * 72.0), subtitle); */
  fprintf(output_fp, "%f 2 div 72 mul XScale div (%s) stringwidth pop 2 div sub %f TAO YScale div m (%s) s\n", 
	  xdim, subtitle, ((ydim + 0.25) * 72.0), subtitle);


}


float
fit(int long_tick, int short_tick, float *long_dim, float max_long, float max_short)
{
  float short_dim;


  short_dim = (*long_dim) * short_tick / long_tick;
  
  while (*long_dim > max_long || short_dim > max_short)
    {
      *long_dim -= 0.1;
      short_dim = (*long_dim) * short_tick / long_tick;
    }
  
  return short_dim;
}


void
usage(char *progname)
{

  fprintf(stderr, "Usage: %s [-xlabel string] [-ylabel string] [-title string]\n"         /* ITS4: ignore */
	  "\t\t[-subtitle string] [-xmax #] [-ymax #] [-xmin #]\n"
	  "\t\t[-ymin #] [-xtick #] [-ytick #] [-pagewidth #]\n"
	  "\t\t[-pageheight #] [-landscape] [-nolandscape] [-grid]\n"
	  "\t\t[-nogrid] [-equal] [-noequal] [-input filename]\n"
	  "\t\t[-output filename] [-font fontname] [-glabel string]\n"
	  "\t\t[-shade] [-noshade] [-color] [-nocolor] [-epsf]\n"
	  "\t\t[-lines] [-nolines] [-dots] [-nodots] [-help] [-version]\n",
	  progname);
}


#ifndef MAX_PS_STACK
#define MAX_PS_STACK 1000
#endif

#ifndef MAX_GREY
#define MAX_GREY	0.8
#endif

/* with the exception of the first time, graph_no must always be odd numbered */

void
make_graph(FILE *input_fp, FILE *output_fp, float xmax, float xmin, int shade_graph, 
	   char *graph_label, int graph_no, int pow_two, char *first_data_point,
	   int x_precis, int y_precis, char x_form, char y_form, int connect_lines, int make_dots)
{
  int s;
  float x;
  float y;
  int i = 1;
  char string[16];
  float gray_set;
  char scan_form_string[8];
  char print_form_string[64];


  if (graph_no == 0)
    {
      gray_set = (float) 0;
      if (shade_graph == TRUE)
	{
	  graph_no = 1;
	  pow_two = 1;
	}
    }
  else
    {
      if (pow_two == 0)
	{
	  pow_two = 1;
	}
      if (shade_graph == TRUE)
	{
	  if (graph_no > pow_two)
	    {
	      while (graph_no > pow_two)
		{
		  pow_two = pow_two * 2;
		}
	      graph_no = 1;
	    }
	  gray_set = (graph_no * MAX_GREY) / pow_two;
	  graph_no += 2;
	}
      else
	{
	  gray_set = (graph_no * MAX_GREY) / pow_two;
	}
    }

  fprintf(output_fp, "%f sg\n", gray_set);		/* set the shading of the graph */
  
  sprintf(scan_form_string, "%%g:%%g");

  sprintf(print_form_string, "%%.*%c convx %%.*%c convy m\n", x_form, y_form);

/**/
  if (first_data_point != NULL)
    {
      while ((s =  sscanf(first_data_point, scan_form_string, &x, &y)) != EOF && s == 2)
	{
	  if ((xmin < xmax) ? (x <= xmax && x >= xmin ): (x <= xmin && x >= xmax ))
	    {
	      fprintf(output_fp, print_form_string, x_precis, x, y_precis, y);
	      break;
	    }
	}
    }
  else
    {
/**/

/* this gets the first point and moves us there */
  while ( (s =  fscanf(input_fp, scan_form_string, &x, &y)) != EOF && s == 2)
    {
      if ((xmin < xmax) ? (x <= xmax && x >= xmin ): (x <= xmin && x >= xmax ))
	{
	  fprintf(output_fp, print_form_string, x_precis, x, y_precis, y);
	  break;
	}
    }
/**/
  }
/**/

  sprintf(print_form_string, "%%.*%c convx %%.*%c convy %s%s\n", x_form, y_form, 
	  (connect_lines) ? "l" : "m",
	  (make_dots) ? " c" : "");

  while ( (s =  fscanf(input_fp, scan_form_string, &x, &y)) != EOF && s == 2)
    {
      if ((xmin < xmax) ? (x <= xmax && x >= xmin ): (x <= xmin && x >= xmax ))
	{
	  fprintf(output_fp, print_form_string, x_precis, x, y_precis, y);
	}
      if (++i == MAX_PS_STACK)
	{
	  fprintf(output_fp, "currentpoint st m\n");                        /* ITS4: ignore */
	  i = 1;
	}
    }

  if (i != 1)
    {
      if (*graph_label != '\0')
	{
	  fprintf(output_fp, "currentpoint st m 0 GlO rmoveto 0 sg (%s) s\n", graph_label);
	}
      else
	{
	  fprintf(output_fp, "currentpoint st m\n");                       /* ITS4: ignore */
	}
    }

 /* check to see if we need to draw another graph */
  if (s != EOF)
    {
      for (i = 0 ; ((string[i] = fgetc(input_fp)) != EOF) &&  i < 15; i++)    /* ITS4: ignore fgetc */
	{
	  if (string[i] == '\n')
	    {
	      break;
	    }
	}	
      string[i] = '\0';

      if (!strncmp("new", string, 3))
	{
	  int cf;
	  int ac;
	  char *new_graph_label;
	  int glabel_count = FALSE;
	  char **my_argv;
	  extern int joptind;
	  extern char *joptarg;
	  int tmp_shade;

	  
	  tmp_shade = shade_graph;
      
	  if (i == 15)
	    {
	      while ((s = fgetc(input_fp)) != EOF && s != '\n')             /* ITS4: ignore fgetc */
		{
		}
	    }

	  my_argv = (char **) malloc(256 * (sizeof (int)));
	  ac = 0;

	  my_argv = get_my_argv(my_argv, input_fp, &ac);
	  
	  joptind = 0;
	  while ((cf = jgetopt(ac, my_argv, "glabel: shade noshade")) != EOF) 
	    {
	      switch (cf) 
		{
		case 0:
		  if (glabel_count == FALSE) 
		    {
		      new_graph_label = (char *) malloc(strlen(joptarg));
		      strcpy(new_graph_label, joptarg);                   /* ITS4: ignore */
		      glabel_count = TRUE;
		    } 
		  break;
		case 1:
		  if (tmp_shade == FALSE)
		    {
		      shade_graph = TRUE;
		      if (graph_no == 0)
			{
			  graph_no = 1;
			  pow_two = 1;
			}
		      else
			{
			  graph_no += 2;
			  pow_two = pow_two * 2;
			}
		    }
		  break;
		case 2:
		  if (tmp_shade == TRUE)
		    {
		      shade_graph = FALSE;
		    }
		  break;
		default:
		  break;
		}
	    }

	  if (glabel_count == FALSE) 
	    {
	      new_graph_label = "\0";
	    } 

	  make_graph(input_fp, output_fp, xmax, xmin, shade_graph, new_graph_label,
		     graph_no, pow_two, NULL, x_precis, y_precis, x_form, y_form, connect_lines, 
		     make_dots);
	}
    }

  return;
}


char **get_my_argv(char **passed_argv, FILE *input_fp, int *ac)
{

  int s;
  char **ma_argv;
  char **ta_argv;

  ta_argv = passed_argv;
  ma_argv = ta_argv;

  if (*ac > 0)
    ta_argv += *ac;

  while ((s = getc(input_fp)) != EOF)                                 /* ITS4: ignore getc */
    {
      if (isdigit(s))
	{
	  ungetc(s, input_fp);
	  break;
	}
      else
	{
	  char *tmp;
	  char *bung;
	  char *argument;
	  char *tmp1;

	  tmp1 = (char *) malloc(128);
	  argument = tmp1;
		      
	  tmp = (char *) malloc(16);
	  bung = tmp;
	  *tmp++ = '-';
	  *tmp++ = s;
	  while((s = getc(input_fp)) != EOF)
	    {
	      if (s == '=' || s == '\n')
		{
		  *tmp = '\0';
		  break;
		}
	      *tmp++ = s;
	    }
	  
	  *ta_argv++ = bung;
	  *ac += 1;

	  switch (s)
	    {
	    case '=':		/* assume we are dealing with only flags with arguments */
	      while((s = getc(input_fp)) != EOF)
		{
		  if (s == '\n')
		    {
		      *tmp1 = '\0';
		      break;
		    }
		  *tmp1++ = s;
		}
	      *ta_argv++ = argument;
	      *ac += 1;
	      break;
	    case '\n':		/* assume we are dealing with only flags with no args */
	      break;
	    }
	}
    }
  
  return ma_argv;
}

char *get_first_line(FILE *input_fp)
{
  char *s;
  char *p;
  int c;

  p = (char *) malloc(BUFSIZ);
  s = p;

  while ((c = getc(input_fp)) != EOF)
    {
      if (c == '\n')
	{
	  *p = '\0';
	  break;
	}
      *p++ = c;
    }

  return s;
}


int get_form(char *first_data_point, int *y_len, char *y_form, int *x_len, char *x_form)
{
  int s = 0;
  int precis = 0;
  int length;
  char *y_point;
  char *x_point;
  char *bung;

  *x_len = 0;
  *y_len = 0;

  length = strlen(first_data_point) + 1;

  y_point = (char *) malloc(length);
  x_point = (char *) malloc(length);
  bung = (char *) malloc(length);

  *x_form = 'f';
  *y_form = 'f';

#ifdef SOLARIS
  bung = (char *) strchr(first_data_point, ':');
#else
  bung = (char *) index(first_data_point, ':');
#endif

  if (bung != NULL)
    {
      strcpy(y_point, bung);                                    /* ITS4: ignore */
      y_point += 1;
      *bung = '\0';

      /* examine the x_point */
#ifdef SOLARIS
      x_point = (char *) strchr(first_data_point, '.');
#else
      x_point = (char *) index(first_data_point, '.');
#endif
      if (x_point != NULL)
	{
	  x_point += 1;
	  *x_len = strlen(x_point);
#ifdef SOLARIS
	  if ((bung = (char *) strchr(x_point, 'E')) != NULL)
#else
	  if ((bung = (char *) index(x_point, 'E')) != NULL)
#endif
	    {
	      *x_form = 'E';
	      *bung = '\0';
	      *x_len = strlen(x_point);
	    }
#ifdef SOLARIS
	  else if ((bung = (char *) strchr(x_point, 'e')) != NULL)
#else
	  else if ((bung = (char *) index(x_point, 'e')) != NULL)
#endif
	    {
	      *x_form = 'e';
	      *bung = '\0';
	      *x_len = strlen(x_point);
	    }
	}

      /* examine the y_point */
#ifdef SOLARIS
      y_point = (char *) strchr(y_point, '.');
#else
      y_point = (char *) index(y_point, '.');
#endif
      if (y_point != NULL)
	{
	  y_point += 1;
	  *y_len = strlen(y_point);
#ifdef SOLARIS
	  if ((bung = (char *) strchr(y_point, 'E')) != NULL)
#else
	  if ((bung = (char *) index(y_point, 'E')) != NULL)
#endif
	    {
	      *y_form = 'E';
	      *bung = '\0';
	      *y_len = strlen(y_point);
	    }
#ifdef SOLARIS
	  else if ((bung = (char *) strchr(y_point, 'e')) != NULL)
#else
	  else if ((bung = (char *) index(y_point, 'e')) != NULL)
#endif
	    {
	      *y_form = 'e';
	      *bung = '\0';
	      *y_len = strlen(y_point);
	    }
	}

      precis = ((x_len >= y_len) ? *x_len : *y_len);
    }

  return s;
}
